function StatusMarker(controller, id, placement) {
	this.controller = controller;
	this.id = id;
	this.position = {
			x: placement.origin.x + (placement.offset.x * StatusMarker.width) - (StatusMarker.width / 2),
			y: placement.origin.y + (placement.offset.y * StatusMarker.height) - (StatusMarker.height / 2),
	};
	this.enabled = true;
	this.shapes = controller.draw.group();
	this.shape = controller.draw
	.rect(StatusMarker.width, StatusMarker.height)
	.radius(StatusMarker.radius)
	.move(this.position.x, this.position.y)
	.attr(StatusMarker.attr.enabled)
	.addTo(this.shapes);
	this.text = controller.draw
	.text('rover ' + this.id)
	.move(this.position.x + StatusMarker.textOffset.x, this.position.y + StatusMarker.textOffset.y)
	.attr(StatusMarker.attr.enabled)
	.attr({'text-anchor': 'middle'})
	.addTo(this.shapes);
}

StatusMarker.width = Controller.width * (1/20);
StatusMarker.height = StatusMarker.width;
StatusMarker.radius = StatusMarker.width / 3;
StatusMarker.textOffset = {
		x: StatusMarker.width / 2,
		y: StatusMarker.height * 1.1,
};

StatusMarker.attr = {};
StatusMarker.attr.enabled = {
		fill: '#00a300',
};
StatusMarker.attr.disabled = {
		fill: '#ccc',
};

StatusMarker.timeout = 6000;

StatusMarker.prototype.onHit = function() {
	this.enabled = false;
	this.updateController();
	this.blink(0);
	var status = this;
	window.setTimeout(function() {
		status.enabled = true;
		status.updateController();
		status.shape.attr(StatusMarker.attr.enabled);
	}, StatusMarker.timeout);
};

StatusMarker.prototype.updateController = function() {
	if (this.controller.id == this.id) {
		this.controller.ready(this.enabled);		
	}
};

StatusMarker.prototype.blink = function(count) {
	var status = this;
	if (count < 3) {
		// Blink three times.
		this.shape.animate(250).fill('#da532c').after(function() {
			this.animate(250).fill('#f5f5f5').after(function() {
				status.blink(count + 1);
			});
		});
	} else {
		// At end, show the marker as disabled.
		this.shape.attr(StatusMarker.attr.disabled);
	}
};

StatusMarker.prototype.show = function() {
	this.shapes.show();
};

StatusMarker.prototype.hide = function() {
	this.shapes.hide();
};


